package net.wanji.opt.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/2/9 8:38
 */
@Data
@NoArgsConstructor
@ApiModel(value = "SchemeEvaluateProblemSchemeVO", description = "问题方案列表元素")
public class SchemeEvaluateProblemSchemeVO {

    @ApiModelProperty(value = "问题次数")
    private Integer problemCounts;

    @ApiModelProperty(value = "问题方案个数")
    private Integer schemeCounts;

    @ApiModelProperty(value = "效率问题个数")
    private Integer efficiencyProblemCounts;

    @ApiModelProperty(value = "均衡问题个数")
    private Integer balanceProblemCounts;

    @ApiModelProperty(value = "安全问题个数")
    private Integer securityProblemCounts;

    @ApiModelProperty(value = "问题方案列表（根据日期）")
    private List<ByDateVO> problemSchemeListByDate;

    @ApiModelProperty(value = "问题方案列表（根据方案）")
    private List<BySchemeVO> problemSchemeListByScheme;

    @NoArgsConstructor
    @Data
    public static class ByDateVO {
        @ApiModelProperty(value = "日期")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "M/d", timezone = "GMT+8")
        private Date problemDate;

        @ApiModelProperty(value = "效率问题个数")
        private Integer efficiencyProblemCounts;

        @ApiModelProperty(value = "均衡问题个数")
        private Integer balanceProblemCounts;

        @ApiModelProperty(value = "安全问题个数")
        private Integer securityProblemCounts;

        @ApiModelProperty(value = "时段方案列表")
        private List<SectionSechemeVO> sectionSechemeList;
    }

    @NoArgsConstructor
    @Data
    public static class SectionSechemeVO {
        @ApiModelProperty(value = "时段")
        private String timeSection;
        @ApiModelProperty(value = "方案名称")
        private String schemeName;
    }

    @NoArgsConstructor
    @Data
    public static class BySchemeVO {
        @ApiModelProperty(value = "方案名称")
        private String schemeName;

        @ApiModelProperty(value = "日期")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "M/d", timezone = "GMT+8")
        private Date problemDate;

        @ApiModelProperty(value = "开始时间")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "HH:mm", timezone = "GMT+8")
        Date startTime;

        @ApiModelProperty(value = "结束时间")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "HH:mm", timezone = "GMT+8")
        Date endTime;
    }
}
